/*
	XmBlackBerry, Copyright (C) 2008-2009 Rick Scott <rwscott@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 2.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
    Copyright 2008, Chris Frey, NetDirect (http://www.netdirect.ca/), GPL v2
    2008/11 - Now compiles on 2.6.26.5: Changes to URB callback mechanism and
	      work queue API.  Also uses new sha1 API.
*/

#ifndef _BLACKBERRY_H
#define _BLACKBERRY_H

#include <linux/version.h>
#include <linux/module.h>
#include <linux/crypto.h>
#include <linux/tty.h>

#include "modem.h"

#define MAX_PACKET (0x400)

struct device_map {
    u_int32_t pin;
    struct blackberry_device *blackberry_device;
    u8 *password_hash;
    int attempt_bootload;
};

struct blackberry_mode {
    char *mode;
    struct tty_struct *tty;
    int open_count;
    struct semaphore sem;
    int socket;
    int connected;
    wait_queue_head_t open_wait;
    u_int8_t buffer[0x4000];
    int num_buffer;
    unsigned int msr;
    unsigned int mcr;
    void (*write)(struct blackberry_mode *mode);
    int (*send_packet)(struct blackberry_mode *mode, const void *data, int len);
};

struct blackberry_device {
    struct usb_device *usb_device;
    struct usb_interface *interface;
    struct kref kref;
    u_int32_t pin;
    u_int32_t hwid;
    int max_packet;
    struct work_struct read_work;
    struct blackberry_mode desktop;
    struct modem modem;

    struct blackberry_mode loader;
    struct blackberry_mode ftp;
    struct blackberry_mode caltask;
    struct blackberry_mode jvmdebug;
    struct blackberry_mode bootload;

    int read_ep;
    int write_ep;
    spinlock_t urb_queue_lock;
    struct list_head urb_queue;
    struct urb *last_urb;
    int attempts_remaining;
    struct urb *rx_urb;
    char *rx_buf;
};
#define to_blackberry_device(d) container_of(d, struct blackberry_device, kref)

extern int debug;
extern int num_devices;
extern struct device_map *device_map;
extern struct workqueue_struct *work_queue;
void blackberry_put(struct blackberry_device *blackberry_device);
struct blackberry_device *blackberry_get(struct blackberry_device *blackberry_device);
void select_mode(struct blackberry_device *blackberry_device, const char *mode);
void open_socket(struct blackberry_device *blackberry_device, int socket);
void close_socket(struct blackberry_device *blackberry_device, int socket);
void reset_device(struct blackberry_device *blackberry_device);
int blackberry_write(struct blackberry_device *blackberry_device, int socket, const void *input, int len, int mem_flags);
int tty_send(struct tty_struct *tty, const unsigned char *buf, int count);
int send_password(struct blackberry_mode *mode, int mem_flags);

int mode_open(struct blackberry_mode *mode, struct tty_struct *tty, struct file *file);
int mode_register_device(struct tty_driver *driver, struct blackberry_mode *mode, int index);
int mode_deregister_device(struct tty_driver *driver, struct blackberry_mode *mode, int index);
struct tty_driver *mode_register(const char *name, int  (*open)(struct tty_struct * tty, struct file * filp));
int mode_deregister(struct tty_driver *driver);
#endif
