/*
	XmBlackBerry, Copyright (C) 2009  Rick Scott <rwscott@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 2.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "config.h"

#include <linux/module.h>	/* Needed by all modules */
#include <linux/kernel.h>	/* Needed for KERN_INFO */
#include <linux/usb.h>
#include <linux/string.h>
#include <linux/tty.h>

#include "bootload.h"
#include "util.h"

static struct tty_driver *bootload_driver = NULL;

static int
bootload_send_packet(struct blackberry_mode *bootload, const void *data, int usb_len)
{
struct blackberry_device *blackberry_device = (bootload && bootload->tty) ? device_map[bootload->tty->index].blackberry_device : NULL;
int ret = 0;

    if (debug >= 2)
    {
	info("%s(%d) - %08x socket = 0x%04x len = %i", __FUNCTION__, __LINE__,
	    blackberry_device ? blackberry_device->pin : (uint32_t)-1,
	    bootload->socket, usb_len);
	dump_hex("--> tty", data, usb_len);
    }
    ret = tty_send(bootload->tty, data, usb_len);
    return(ret);
}

static int
bootload_open(struct tty_struct *tty, struct file *file)
{
int ret = 0;
struct blackberry_device *blackberry_device = tty ? device_map[tty->index].blackberry_device : NULL;
struct blackberry_mode *mode = blackberry_device ? &blackberry_device->bootload : NULL;

    ret = mode_open(mode, tty, file);
    return(ret);
}

int
bootload_register_device(struct blackberry_mode *bootload, int index)
{
int ret = 0;
struct blackberry_device *blackberry_device = container_of(bootload, struct blackberry_device, bootload);

    if (debug >= 1)
    {
	info("%s(%d) - %08x /dev/%s%i", __FUNCTION__, __LINE__,
	    blackberry_device ? blackberry_device->pin : (u_int32_t)-1,
	    bootload_driver->name,
	    index);
    }
    bootload->mode = "RIM-BootLoader";
    bootload->write = NULL;
    bootload->send_packet = bootload_send_packet;
    ret = mode_register_device(bootload_driver, bootload, index);
    return(ret);
}

int
bootload_deregister_device(struct blackberry_mode *bootload, int index)
{
int ret = 0;
struct blackberry_device *blackberry_device = container_of(bootload, struct blackberry_device, bootload);

    if (debug >= 1)
    {
	info("%s(%d) - %08x /dev/%s%i", __FUNCTION__, __LINE__,
	    blackberry_device ? blackberry_device->pin : (u_int32_t)-1,
	    bootload_driver->name,
	    index);
    }
    ret = mode_deregister_device(bootload_driver, bootload, index);
    return(ret);
}

int
bootload_register()
{
int ret = 0;

    if (debug >= 8)
    {
	info("%s(%d)", __FUNCTION__, __LINE__);
    }
    bootload_driver = mode_register("bbootload", bootload_open);
    if (bootload_driver)
    {
    	bootload_driver->driver_name = "RIM Bootloader";
    }
    else
    {
    	ret = -ENOMEM;
	err("%s(%d)", __FUNCTION__, __LINE__);
    }
    return(ret);
}

int
bootload_deregister()
{
int ret = 0;

    if (debug >= 8)
    {
	info("%s(%d)", __FUNCTION__, __LINE__);
    }
    ret = mode_deregister(bootload_driver);
    return(ret);
}
