/*
	XmBlackBerry, Copyright (C) 2009  Rick Scott <rwscott@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 2.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "config.h"

#include <linux/module.h>	/* Needed by all modules */
#include <linux/kernel.h>	/* Needed for KERN_INFO */
#include <linux/usb.h>
#include <linux/string.h>
#include <linux/tty.h>

#include "caltask.h"
#include "util.h"

static struct tty_driver *caltask_driver = NULL;

static void
process_buffer(struct blackberry_mode *caltask)
{
struct blackberry_device *blackberry_device = container_of(caltask, struct blackberry_device, caltask);

    if (debug >= 8)
    {
	info("%s(%d) - %08x num_buffer = %i", __FUNCTION__, __LINE__,
	    blackberry_device ? blackberry_device->pin : (u_int32_t)-1,
	    blackberry_device ? blackberry_device->caltask.num_buffer : -1);
	dump_hex("--", &blackberry_device->caltask.buffer[0], blackberry_device->caltask.num_buffer);
    }

    if (blackberry_device)
    {
	while (caltask->num_buffer >= 2)
	{
	int num;

	    num = 0;
	    num = (num << 8) + caltask->buffer[1];
	    num = (num << 8) + caltask->buffer[0];
	    if (num < sizeof(caltask->buffer) - 2)
	    {
		if (caltask->num_buffer >= 2 + num)
		{
		    if (debug >= 2)
		    {
			info("%s(%d) - %08x num_buffer = %i", __FUNCTION__, __LINE__,
			    blackberry_device ? blackberry_device->pin : (u_int32_t)-1,
			    blackberry_device ? blackberry_device->caltask.num_buffer : -1);
			dump_hex("--> usb", &blackberry_device->caltask.buffer[2], num);
		    }
		    blackberry_write(blackberry_device, caltask->socket, &caltask->buffer[2], num, GFP_ATOMIC);
		    memmove(&caltask->buffer[0], &caltask->buffer[2 + num], caltask->num_buffer - (num + 2));
		    caltask->num_buffer -= (num + 2);
		}
		else
		{
		    /* Not enough data to make a decision yet */
		    break;
		}
	    }
	    else
	    {
		caltask->num_buffer = 0;
		err("%s(%d) - %08x Unreasonable transfer size 0x%04x", __FUNCTION__, __LINE__,
		    blackberry_device ? blackberry_device->pin : (u_int32_t)-1,
		    num);
	    }
	}
    }
    else
    {
	err("%s(%d) - %08x", __FUNCTION__, __LINE__,
	    blackberry_device ? blackberry_device->pin : (u_int32_t)-1);
    }
}

static int
caltask_send_packet(struct blackberry_mode *caltask, const void *data, int usb_len)
{
struct blackberry_device *blackberry_device = (caltask && caltask->tty) ? device_map[caltask->tty->index].blackberry_device : NULL;
int ret = 0;
uint8_t tty_len[2];

    if (debug >= 2)
    {
	info("%s(%d) - %08x socket = 0x%04x len = %i", __FUNCTION__, __LINE__,
	    blackberry_device ? blackberry_device->pin : (uint32_t)-1,
	    caltask->socket, usb_len);
	dump_hex("--> tty", data, usb_len);
    }
    tty_len[0] = (usb_len >> 0) & 0xff;
    tty_len[1] = (usb_len >> 8) & 0xff;
    tty_send(caltask->tty, tty_len, sizeof(tty_len));
    ret = tty_send(caltask->tty, data, usb_len);
    return(ret);
}

static int
caltask_open(struct tty_struct *tty, struct file *file)
{
int ret = 0;
struct blackberry_device *blackberry_device = tty ? device_map[tty->index].blackberry_device : NULL;
struct blackberry_mode *mode = blackberry_device ? &blackberry_device->caltask : NULL;

    ret = mode_open(mode, tty, file);
    return(ret);
}

int
caltask_register_device(struct blackberry_mode *caltask, int index)
{
int ret = 0;
struct blackberry_device *blackberry_device = container_of(caltask, struct blackberry_device, caltask);

    if (debug >= 1)
    {
	info("%s(%d) - %08x /dev/%s%i", __FUNCTION__, __LINE__,
	    blackberry_device ? blackberry_device->pin : (u_int32_t)-1,
	    caltask_driver->name,
	    index);
    }
    caltask->mode = "RIM Cal task";
    //caltask->mode = "Caltask Config";
    //caltask->mode = "Caltask Secure";
    caltask->write = process_buffer;
    caltask->send_packet = caltask_send_packet;
    ret = mode_register_device(caltask_driver, caltask, index);
    return(ret);
}

int
caltask_deregister_device(struct blackberry_mode *caltask, int index)
{
int ret = 0;
struct blackberry_device *blackberry_device = container_of(caltask, struct blackberry_device, caltask);

    if (debug >= 1)
    {
	info("%s(%d) - %08x /dev/%s%i", __FUNCTION__, __LINE__,
	    blackberry_device ? blackberry_device->pin : (u_int32_t)-1,
	    caltask_driver->name,
	    index);
    }
    ret = mode_deregister_device(caltask_driver, caltask, index);
    return(ret);
}

int
caltask_register()
{
int ret = 0;

    if (debug >= 8)
    {
	info("%s(%d)", __FUNCTION__, __LINE__);
    }
    caltask_driver = mode_register("bbcaltask", caltask_open);
    if (caltask_driver)
    {
    	caltask_driver->driver_name = "RIM Calibration Task";
    }
    else
    {
    	ret = -ENOMEM;
	err("%s(%d)", __FUNCTION__, __LINE__);
    }
    return(ret);
}

int
caltask_deregister()
{
int ret = 0;

    if (debug >= 8)
    {
	info("%s(%d)", __FUNCTION__, __LINE__);
    }
    ret = mode_deregister(caltask_driver);
    return(ret);
}
