/*
	XmBlackBerry, Copyright (C) 2008  Rick Scott <rwscott@users.sourceforge.net>
	Copyright 2008, Chris Frey, NetDirect Inc. (http://www.netdirect.ca/)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 2.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "config.h"

#include <linux/mm.h>
#include <linux/err.h>
#include <linux/crypto.h>
#include <linux/usb.h>
#include <asm/scatterlist.h>
#ifdef HAVE_LINUX_SCATTERLIST_H
#include <linux/scatterlist.h>
#else
static inline void sg_set_buf(struct scatterlist *sg, void *buf,
			      unsigned int buflen)
{
	sg->page = virt_to_page(buf);
	sg->offset = offset_in_page(buf);
	sg->length = buflen;
}

static inline void sg_init_one(struct scatterlist *sg, void *buf,
			       unsigned int buflen)
{
	memset(sg, 0, sizeof(*sg));
	sg_set_buf(sg, buf, buflen);
}
#endif

#include "crypto.h"
#include "util.h"

#ifdef HAVE_HASH_DESC
static struct crypto_hash *crypto_hash = NULL;
static struct hash_desc crypto_hash_desc;
#else
static struct crypto_tfm *crypto_tfm = NULL;
#endif

static int sha1_available = 0;

int
blackberry_sha1_init(void)
{
#ifdef HAVE_HASH_DESC
    	crypto_hash = crypto_alloc_hash("sha1", 0, CRYPTO_ALG_ASYNC);
	if( IS_ERR(crypto_hash) )
	{
	    info("%s(%d) - failed to load sha1", __FUNCTION__, __LINE__);
	}
	else
	{
	    crypto_hash_desc.tfm = crypto_hash;
	    crypto_hash_desc.flags = 0;

	    // success!
	    sha1_available = 1;
	}

#else
    	crypto_tfm = crypto_alloc_tfm("sha1", 0);
    	if (!crypto_tfm)
    	{
	    info("%s(%d) - failed to load sha1", __FUNCTION__, __LINE__);
    	}
    	else
    	{
	    sha1_available = 1;
	}
#endif
	return(sha1_available);
}

void
blackberry_sha1_dnit(void)
{
#ifdef HAVE_HASH_DESC
    if (crypto_hash)
    {
    	crypto_free_hash(crypto_hash);
    }
#else
    if (crypto_tfm)
    {
    	crypto_free_tfm(crypto_tfm);
    }
#endif
}

int
blackberry_sha1_avail(void)		// returns 1 if sha1 is available
{
    return sha1_available;
}

unsigned int
blackberry_sha1_digestsize(void)
{
#ifdef HAVE_HASH_DESC
    return crypto_hash ? crypto_hash_digestsize(crypto_hash) : 0;
#else
    return crypto_tfm ? crypto_tfm_alg_digestsize(crypto_tfm) : 0;
#endif
}

void
blackberry_sha1_sum(const void *input, int len, unsigned char *hashout)
{
struct scatterlist sg;

#ifdef HAVE_HASH_DESC
    int ret;

    sg_init_one(&sg, input, len);

    ret = crypto_hash_digest(&crypto_hash_desc, &sg, len, hashout);
    if( ret )
    {
	sha1_available = 0;
	return;
    }
#else
    sg_init_one(&sg,
	(u8 *)input, len);
    crypto_digest_init(crypto_tfm);
    crypto_digest_update(crypto_tfm, &sg, 1);
    crypto_digest_final(crypto_tfm, hashout);
#endif
}
