/*
	XmBlackBerry, Copyright (C) 2009  Rick Scott <rwscott@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 2.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "config.h"

#include <linux/module.h>	/* Needed by all modules */
#include <linux/kernel.h>	/* Needed for KERN_INFO */
#include <linux/usb.h>
#include <linux/string.h>
#include <linux/tty.h>

#include "ftp.h"
#include "util.h"

static struct tty_driver *ftp_driver = NULL;

static int
ftp_send_packet(struct blackberry_mode *ftp, const void *data, int usb_len)
{
struct blackberry_device *blackberry_device = (ftp && ftp->tty) ? device_map[ftp->tty->index].blackberry_device : NULL;
int ret = 0;

    if (debug >= 2)
    {
	info("%s(%d) - %08x socket = 0x%04x len = %i", __FUNCTION__, __LINE__,
	    blackberry_device ? blackberry_device->pin : (uint32_t)-1,
	    ftp->socket, usb_len);
	dump_hex("--> tty", data, usb_len);
    }
    ret = tty_send(ftp->tty, data, usb_len);
    return(ret);
}

static int
ftp_open(struct tty_struct *tty, struct file *file)
{
int ret = 0;
struct blackberry_device *blackberry_device = tty ? device_map[tty->index].blackberry_device : NULL;
struct blackberry_mode *mode = blackberry_device ? &blackberry_device->ftp : NULL;

    ret = mode_open(mode, tty, file);
    return(ret);
}

int
ftp_register_device(struct blackberry_mode *ftp, int index)
{
int ret = 0;
struct blackberry_device *blackberry_device = container_of(ftp, struct blackberry_device, ftp);

    if (debug >= 1)
    {
	info("%s(%d) - %08x /dev/%s%i", __FUNCTION__, __LINE__,
	    blackberry_device ? blackberry_device->pin : (u_int32_t)-1,
	    ftp_driver->name,
	    index);
    }
    ftp->mode = "RIM_DFTP";
    ftp->write = NULL;
    ftp->send_packet = ftp_send_packet;
    ret = mode_register_device(ftp_driver, ftp, index);
    return(ret);
}

int
ftp_deregister_device(struct blackberry_mode *ftp, int index)
{
int ret = 0;
struct blackberry_device *blackberry_device = container_of(ftp, struct blackberry_device, ftp);

    if (debug >= 1)
    {
	info("%s(%d) - %08x /dev/%s%i", __FUNCTION__, __LINE__,
	    blackberry_device ? blackberry_device->pin : (u_int32_t)-1,
	    ftp_driver->name,
	    index);
    }
    ret = mode_deregister_device(ftp_driver, ftp, index);
    return(ret);
}

int
ftp_register()
{
int ret = 0;

    if (debug >= 8)
    {
	info("%s(%d)", __FUNCTION__, __LINE__);
    }
    ftp_driver = mode_register("bbftp", ftp_open);
    if (ftp_driver)
    {
    	ftp_driver->driver_name = "RIM DFTP";
    }
    else
    {
    	ret = -ENOMEM;
	err("%s(%d)", __FUNCTION__, __LINE__);
    }
    return(ret);
}

int
ftp_deregister()
{
int ret = 0;

    if (debug >= 8)
    {
	info("%s(%d)", __FUNCTION__, __LINE__);
    }
    ret = mode_deregister(ftp_driver);
    return(ret);
}
