/*
	XmBlackBerry, Copyright (C) 2009  Rick Scott <rwscott@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 2.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "config.h"

#include <linux/module.h>	/* Needed by all modules */
#include <linux/kernel.h>	/* Needed for KERN_INFO */
#include <linux/usb.h>
#include <linux/string.h>
#include <linux/tty.h>

#include "jvmdebug.h"
#include "util.h"

static struct tty_driver *jvmdebug_driver = NULL;

static int
jvmdebug_send_packet(struct blackberry_mode *jvmdebug, const void *data, int usb_len)
{
struct blackberry_device *blackberry_device = (jvmdebug && jvmdebug->tty) ? device_map[jvmdebug->tty->index].blackberry_device : NULL;
int ret = 0;

    if (debug >= 2)
    {
	info("%s(%d) - %08x socket = 0x%04x len = %i", __FUNCTION__, __LINE__,
	    blackberry_device ? blackberry_device->pin : (uint32_t)-1,
	    jvmdebug->socket, usb_len);
	dump_hex("--> tty", data, usb_len);
    }
    ret = tty_send(jvmdebug->tty, data, usb_len);
    return(ret);
}

static int
jvmdebug_open(struct tty_struct *tty, struct file *file)
{
int ret = 0;
struct blackberry_device *blackberry_device = tty ? device_map[tty->index].blackberry_device : NULL;
struct blackberry_mode *mode = blackberry_device ? &blackberry_device->jvmdebug : NULL;

    ret = mode_open(mode, tty, file);
    return(ret);
}

int
jvmdebug_register_device(struct blackberry_mode *jvmdebug, int index)
{
int ret = 0;
struct blackberry_device *blackberry_device = container_of(jvmdebug, struct blackberry_device, jvmdebug);

    if (debug >= 1)
    {
	info("%s(%d) - %08x /dev/%s%i", __FUNCTION__, __LINE__,
	    blackberry_device ? blackberry_device->pin : (u_int32_t)-1,
	    jvmdebug_driver->name,
	    index);
    }
    jvmdebug->mode = "RIM_JVMDebug";
    jvmdebug->write = NULL;
    jvmdebug->send_packet = jvmdebug_send_packet;
    ret = mode_register_device(jvmdebug_driver, jvmdebug, index);
    return(ret);
}

int
jvmdebug_deregister_device(struct blackberry_mode *jvmdebug, int index)
{
int ret = 0;
struct blackberry_device *blackberry_device = container_of(jvmdebug, struct blackberry_device, jvmdebug);

    if (debug >= 1)
    {
	info("%s(%d) - %08x /dev/%s%i", __FUNCTION__, __LINE__,
	    blackberry_device ? blackberry_device->pin : (u_int32_t)-1,
	    jvmdebug_driver->name,
	    index);
    }
    ret = mode_deregister_device(jvmdebug_driver, jvmdebug, index);
    return(ret);
}

int
jvmdebug_register()
{
int ret = 0;

    if (debug >= 8)
    {
	info("%s(%d)", __FUNCTION__, __LINE__);
    }
    jvmdebug_driver = mode_register("bbjvmdebug", jvmdebug_open);
    if (jvmdebug_driver)
    {
    	jvmdebug_driver->driver_name = "RIM JVM Debugger";
    }
    else
    {
    	ret = -ENOMEM;
	err("%s(%d)", __FUNCTION__, __LINE__);
    }
    return(ret);
}

int
jvmdebug_deregister()
{
int ret = 0;

    if (debug >= 8)
    {
	info("%s(%d)", __FUNCTION__, __LINE__);
    }
    ret = mode_deregister(jvmdebug_driver);
    return(ret);
}
