/*
	XmBlackBerry, Copyright (C) 2008  Rick Scott <rwscott@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, version 2.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/** \file
    \brief Some generic helper stuff.
*/
#include "config.h"

#include <linux/kernel.h>
#include <linux/string.h>
#include <linux/ctype.h>

#include "util.h"

/**
	Display a pretty hex dump of some data
*/
#ifdef HAVE_PRINT_HEX_DUMP_BYTES
void
dump_hex(const char *prefix, const void *input, int len)
{
    print_hex_dump(KERN_INFO, prefix, DUMP_PREFIX_OFFSET, 16, 1, input, len, 1);
}
#else
void
dump_hex(const char *prefix, const void *input, int len)
{
const unsigned char *data = input;
int i, j;
char *sep = "";

    printk(KERN_INFO "%s%s", prefix, strlen(prefix) > 0 ? " " : "");
    for (i = 0; i < len && data; i++)
    {
	printk("%s%02x", sep, data[i] & 0xff);
	sep = " ";
	if (!((i + 1) % 8)) printk("%s", sep);
	if (!((i + 1) % 16))
	{
	int j;

	    printk("%s", sep);
	    for (j = i - 16; j < i; j++)
	    {
		printk("%c", isprint(data[j + 1]) ? data[j + 1] : '.');
		if (!((j + 2) % 8)) printk("%s", sep);
	    }
	    if (i + 1 < len)
	    {
		printk("\n%*s%s", strlen(prefix), "", strlen(prefix) > 0 ? " " : "");
	    }
	    sep = "";
	}
    }
    j = i % 16;
    if (j > 0)
    {
	for (i = j; i < 16; i++)
	{
	    printk(" --");
	    if (!((i + 1) % 8)) printk("%s", sep);
	}
	printk(" ");
	for (i = len - j; i < len; i++)
	{
	    printk("%c", isprint(data[i]) ? data[i] : '.');
	    if (!((i + 2) % 8)) printk(" ");
	}
    }
    printk("\n");
}
#endif
